// SimpleDataType.java
//
// (c) 1999-2001 PAL Development Core Team
//
// This package may be distributed under the
// terms of the Lesser GNU General Public License (LGPL)


// Known bugs and limitations:
// - all states must have a non-negative value 0..getNumStates()-1
// - ? (unknown state) has value getNumStates()


package pal.datatype;

import java.io.Serializable;

/**
 * interface for sequence data types
 *
 * @version $Id: SimpleDataType.java,v 1.7 2001/11/26 03:58:59 matt Exp $
 *
 * @author Alexei Drummond
 */
public abstract class SimpleDataType implements DataType
{
	private static final long serialVersionUID=7902613264354545217L;

	//serialver -classpath ./classes pal.datatype.SimpleDataType

	private void writeObject(java.io.ObjectOutputStream out) throws java.io.IOException {
		out.writeByte(1); //Version number
	}

	private void readObject(java.io.ObjectInputStream in) throws java.io.IOException, ClassNotFoundException{
		byte version = in.readByte();
		switch(version) {
			default : {
				break;
			}
		}
	}

	/**
	 * returns true if this state is an ambiguous state.
	 */
	public final boolean isAmbiguousState(int state) {
		return false;
	}

	/**
	 * returns an array containing the non-ambiguous states that this state represents.
	 */
	public final int[] getSimpleStates(int state) {
		int[] simple = new int[1];
		simple[0] = state;
		return simple;
	}

	/**
	 * get number of unique non-ambiguous states
	 *
	 * @return number of unique states
	 */
	public final int getNumSimpleStates() {
		return getNumStates();
	}


		/**
	 * @return true if this state is a gap
	 */
	public boolean isGapChar(char c) {
		return isUnknownState(getState(c));
	}
			/**
	 * @return true if this state is a gap
	 */
	public boolean isUnknownChar(char c) {
		return isUnknownState(getState(c));
	}
		/**
	 * @retrun true if this state is an unknown state
	 */
	public boolean isUnknownState(int state) {
		return(state>=getNumStates())||(state<0);
	}
}
